@@ -1,7 +1,7 @@
 //
 // HTML parsing routines for HTMLDOC, a HTML document processing program.
 //
-// Copyright 2011-2024 by Michael R Sweet.
+// Copyright 2011-2025 by Michael R Sweet.
 // Copyright 1997-2010 by Easy Software Products.  All rights reserved.
 //
 // This program is free software.  Distribution and use rights are outlined in
@@ -9,7 +9,9 @@
 //
 
 #include "htmldoc.h"
-#include <cups/http.h>
+#ifdef HAVE_LIBCUPS
+#  include <cups/http.h>
+#endif // HAVE_LIBCUPS
 #include <ctype.h>
 
 
@@ -120,6 +122,7 @@ const char	*_htmlMarkups[] =
 
 const char	*_htmlCurrentFile = "UNKNOWN";
 					// Current file
+int		_htmlCurrentLevel = 0;	// Current include level
 const char	*_htmlData = HTML_DATA;	// Data directory
 float		_htmlPPI = 80.0f;	// Image resolution
 int		_htmlGrayscale = 0;	// Grayscale output?
@@ -237,8 +240,8 @@ static int	compare_variables(var_t *v0, var_t *v1);
 static int	compare_markups(uchar **m0, uchar **m1);
 static void	delete_node(tree_t *t);
 static void	insert_space(tree_t *parent, tree_t *t);
-static int	parse_markup(tree_t *t, FILE *fp, int *linenum);
-static int	parse_variable(tree_t *t, FILE *fp, int *linenum);
+static int	parse_markup(tree_t *t, FILE *fp, long *linenum);
+static int	parse_variable(tree_t *t, FILE *fp, long *linenum);
 static int	compute_size(tree_t *t);
 static int	compute_color(tree_t *t, uchar *color);
 static int	get_alignment(tree_t *t);
@@ -274,6 +277,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
              FILE       *fp,		// I - File pointer
 	     const char *base)		// I - Base directory for file
 {
+  int		depth;			// Current depth in tree
   int		ch;			// Character from file
   uchar		*ptr,			// Pointer in string
 		entity[16],		// Character entity name (&#nnn; or &name;)
@@ -292,7 +296,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 		*type,			// Type for EMBED tag
 		*span;			// Value for SPAN tag
   int		sizeval;		// Size value from FONT tag
-  int		linenum;		// Line number in file
+  long		linenum;		// Line number in file
   static uchar	s[10240];		// String from file
   static int	have_whitespace = 0;	// Non-zero if there was leading whitespace
 
@@ -300,6 +304,17 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
   DEBUG_printf(("htmlReadFile(parent=%p, fp=%p, base=\"%s\")\n",
                 (void *)parent, (void *)fp, base ? base : "(null)"));
 
+  if (_htmlCurrentLevel >= MAX_INCLUDES)
+  {
+    progress_error(HD_ERROR_NESTING_ERROR, "Too many levels of embedded HTML files (%d)", _htmlCurrentLevel);
+    return (NULL);
+  }
+
+  _htmlCurrentLevel ++;
+
+  for (depth = 0, tree = parent; tree; tree = tree->parent)
+    depth ++;
+
 #ifdef DEBUG
   indent[0] = '\0';
 #endif // DEBUG
@@ -380,8 +395,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
         // Sigh...  "<" followed by anything but an element name is
 	// invalid HTML, but so many people have asked for this to
 	// be supported that we have added this hack...
-	progress_error(HD_ERROR_HTML_ERROR, "Unquoted < on line %d of %s.",
-	               linenum, _htmlCurrentFile);
+	progress_error(HD_ERROR_HTML_ERROR, "Unquoted < on line %ld of %s.", linenum, _htmlCurrentFile);
 
 	if (ch == '\n')
 	  linenum ++;
@@ -415,9 +429,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	if (parse_markup(t, fp, &linenum) == MARKUP_ERROR)
 	{
 #ifndef DEBUG
-          progress_error(HD_ERROR_READ_ERROR,
-                         "Unable to parse HTML element on line %d of %s!",
-			 linenum, _htmlCurrentFile);
+          progress_error(HD_ERROR_READ_ERROR, "Unable to parse HTML element on line %ld of %s.", linenum, _htmlCurrentFile);
 #endif // !DEBUG
 
           delete_node(t);
@@ -434,7 +446,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	if (ch == '/')
 	{
 	  // Close markup; find matching markup...
-          for (temp = parent; temp != NULL; temp = temp->parent)
+          for (temp = parent, depth --; temp != NULL; temp = temp->parent, depth --)
           {
             if (temp->markup == t->markup)
             {
@@ -466,7 +478,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	else if (t->markup == MARKUP_EMBED)
 	{
 	  // Close any text blocks...
-          for (temp = parent; temp != NULL; temp = temp->parent)
+          for (temp = parent, depth --; temp != NULL; temp = temp->parent, depth --)
           {
             if (isblock(temp->markup) || islentry(temp->markup))
             {
@@ -481,7 +493,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	}
 	else if (issuper(t->markup))
 	{
-          for (temp = parent; temp != NULL; temp = temp->parent)
+          for (temp = parent, depth --; temp != NULL; temp = temp->parent, depth --)
           {
 	    if (istentry(temp->markup) || temp->markup == MARKUP_EMBED)
 	    {
@@ -492,7 +504,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	}
 	else if (islist(t->markup))
 	{
-          for (temp = parent; temp != NULL; temp = temp->parent)
+          for (temp = parent, depth --; temp != NULL; temp = temp->parent, depth --)
           {
             if (isblock(temp->markup))
 	    {
@@ -507,7 +519,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	}
 	else if (islentry(t->markup))
 	{
-          for (temp = parent; temp != NULL; temp = temp->parent)
+          for (temp = parent, depth --; temp != NULL; temp = temp->parent, depth --)
           {
             if (islentry(temp->markup))
             {
@@ -522,7 +534,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	}
 	else if (isblock(t->markup))
 	{
-          for (temp = parent; temp != NULL; temp = temp->parent)
+          for (temp = parent, depth --; temp != NULL; temp = temp->parent, depth --)
           {
             if (isblock(temp->markup))
             {
@@ -537,7 +549,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	}
 	else if (t->markup == MARKUP_THEAD || t->markup == MARKUP_TBODY || t->markup == MARKUP_TFOOT)
 	{
-          for (temp = parent; temp != NULL; temp = temp->parent)
+          for (temp = parent, depth --; temp != NULL; temp = temp->parent, depth --)
           {
 	    if (temp->markup == MARKUP_TABLE || temp->markup == MARKUP_EMBED)
 	    {
@@ -548,7 +560,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	}
 	else if (t->markup == MARKUP_TR)
 	{
-          for (temp = parent; temp != NULL; temp = temp->parent)
+          for (temp = parent, depth --; temp != NULL; temp = temp->parent, depth --)
           {
             if (temp->markup == MARKUP_TR)
             {
@@ -563,7 +575,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	}
 	else if (istentry(t->markup))
 	{
-          for (temp = parent; temp != NULL; temp = temp->parent)
+          for (temp = parent, depth --; temp != NULL; temp = temp->parent, depth --)
           {
             if (istentry(temp->markup))
             {
@@ -576,15 +588,11 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	        // Strictly speaking, this is an error - TD/TH can only
 		// be found under TR, but web browsers automatically
 		// inject a TR...
-		progress_error(HD_ERROR_HTML_ERROR,
-		               "No TR element before %s element on line %d of %s.",
-			       _htmlMarkups[t->markup], linenum,
-			       _htmlCurrentFile);
+		progress_error(HD_ERROR_HTML_ERROR, "No TR element before %s element on line %ld of %s.", _htmlMarkups[t->markup], linenum, _htmlCurrentFile);
 
                 parent = htmlAddTree(temp, MARKUP_TR, NULL);
 		prev   = NULL;
-		DEBUG_printf(("%str (inserted) under %s, line %d\n", indent,
-		              _htmlMarkups[temp->markup], linenum));
+		DEBUG_printf(("%str (inserted) under %s, line %ld\n", indent, _htmlMarkups[temp->markup], linenum));
 	      }
 
 	      temp = NULL;
@@ -618,13 +626,8 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	      temp->markup != MARKUP_TR)
 	  {
 	    // Log this condition as an error...
-	    progress_error(HD_ERROR_HTML_ERROR,
-	                   "No /%s element before %s element on line %d of %s.",
-	                   _htmlMarkups[temp->markup],
-			   _htmlMarkups[t->markup], linenum, _htmlCurrentFile);
-	    DEBUG_printf(("%sNo /%s element before %s element on line %d.\n",
-	                  indent, _htmlMarkups[temp->markup],
-			  _htmlMarkups[t->markup], linenum));
+	    progress_error(HD_ERROR_HTML_ERROR, "No /%s element before %s element on line %ld of %s.", _htmlMarkups[temp->markup], _htmlMarkups[t->markup], linenum, _htmlCurrentFile);
+	    DEBUG_printf(("%sNo /%s element before %s element on line %ld.\n", indent, _htmlMarkups[temp->markup], _htmlMarkups[t->markup], linenum));
 	  }
 
 #ifdef DEBUG
@@ -641,6 +644,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	  // the root node created by the caller...
           if (temp->parent)
 	  {
+	    depth --;
 	    parent = temp->parent;
             prev   = parent->last_child;
 	  }
@@ -695,11 +699,8 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	  if (t->markup != MARKUP_UNKNOWN &&
 	      t->markup != MARKUP_COMMENT)
 	  {
-	    progress_error(HD_ERROR_HTML_ERROR,
-	                   "Dangling /%s element on line %d of %s.",
-			   _htmlMarkups[t->markup], linenum, _htmlCurrentFile);
-	    DEBUG_printf(("%sDangling /%s element on line %d.\n",
-			  indent, _htmlMarkups[t->markup], linenum));
+	    progress_error(HD_ERROR_HTML_ERROR, "Dangling /%s element on line %ld of %s.", _htmlMarkups[t->markup], linenum, _htmlCurrentFile);
+	    DEBUG_printf(("%sDangling /%s element on line %ld.\n", indent, _htmlMarkups[t->markup], linenum));
           }
 
 	  delete_node(t);
@@ -733,8 +734,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
           *eptr = '\0';
           if (!ch)
 	  {
-	    progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %d of %s.",
-	                   linenum, _htmlCurrentFile);
+	    progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %ld of %s.", linenum, _htmlCurrentFile);
 
             if (ptr < (s + sizeof(s) - 1))
 	      *ptr++ = '&';
@@ -743,9 +743,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	  }
 	  else if ((ch = iso8859(entity)) == 0)
 	  {
-	    progress_error(HD_ERROR_HTML_ERROR,
-	                   "Unknown character entity \"&%s;\" on line %d of %s.",
-	                   entity, linenum, _htmlCurrentFile);
+	    progress_error(HD_ERROR_HTML_ERROR, "Unknown character entity \"&%s;\" on line %ld of %s.", entity, linenum, _htmlCurrentFile);
 
             if (ptr < (s + sizeof(s) - 1))
 	      *ptr++ = '&';
@@ -787,7 +785,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
       t->markup = MARKUP_NONE;
       t->data   = (uchar *)hd_strdup((char *)s);
 
-      DEBUG_printf(("%sfragment \"%s\", line %d\n", indent, s, linenum));
+      DEBUG_printf(("%sfragment \"%s\", line %ld\n", indent, s, linenum));
     }
     else
     {
@@ -822,8 +820,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 
           if (!ch)
 	  {
-	    progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %d of %s.",
-	                   linenum, _htmlCurrentFile);
+	    progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %ld of %s.", linenum, _htmlCurrentFile);
 
             if (ptr < (s + sizeof(s) - 1))
 	      *ptr++ = '&';
@@ -832,9 +829,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	  }
 	  else if ((ch = iso8859(entity)) == 0)
 	  {
-	    progress_error(HD_ERROR_HTML_ERROR,
-	                   "Unknown character entity \"&%s;\" on line %d of %s.",
-	                   entity, linenum, _htmlCurrentFile);
+	    progress_error(HD_ERROR_HTML_ERROR, "Unknown character entity \"&%s;\" on line %ld of %s.", entity, linenum, _htmlCurrentFile);
 
             if (ptr < (s + sizeof(s) - 1))
 	      *ptr++ = '&';
@@ -878,8 +873,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
       t->markup = MARKUP_NONE;
       t->data   = (uchar *)hd_strdup((char *)s);
 
-      DEBUG_printf(("%sfragment \"%s\" (len=%d), line %d\n", indent, s,
-                    (int)(ptr - s), linenum));
+      DEBUG_printf(("%sfragment \"%s\" (len=%d), line %ld\n", indent, s, (int)(ptr - s), linenum));
     }
 
     // If the parent tree pointer is not null and this is the first
@@ -1542,11 +1536,20 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
       strlcat((char *)indent, "    ", sizeof(indent));
 #endif // DEBUG
 
+      depth ++;
       parent = t;
       prev   = NULL;
+
+      if (depth >= MAX_DEPTH)
+      {
+        progress_error(HD_ERROR_NESTING_ERROR, "Document elements too deeply nested (%d levels).", depth);
+        return (NULL);
+      }
     }
   }
 
+  _htmlCurrentLevel --;
+
   return (tree);
 }
 
@@ -2114,8 +2117,9 @@ uchar *				// O - Content string
 htmlGetMeta(tree_t *tree,	// I - Document tree
             uchar  *name)	// I - Metadata name
 {
-  uchar	*tname,			// Name value from tree entry
-	*tcontent;		// Content value from tree entry
+  tree_t	*top = tree;	// Top of document tree
+  uchar		*tname,		// Name value from tree entry
+		*tcontent;	// Content value from tree entry
 
 
   while (tree != NULL)
@@ -2133,13 +2137,8 @@ htmlGetMeta(tree_t *tree,	// I - Document tree
       return (tcontent);
     }
 
-    // Check child entries...
-    if (tree->child != NULL)
-      if ((tcontent = htmlGetMeta(tree->child, name)) != NULL)
-        return (tcontent);
-
     // Next tree entry...
-    tree = tree->next;
+    tree = htmlWalkNext(top, tree);
   }
 
   return (NULL);
@@ -2508,6 +2507,55 @@ htmlSetTextColor(uchar *color)	// I - Text color
 }
 
 
+//
+// 'htmlWalkNext()' - "Walk" to the next logical node in the tree.
+//
+
+tree_t *				// O - Next logical node or `NULL` if none
+htmlWalkNext(tree_t *doc,		// I - Top of document
+             tree_t *tree,		// I - Current node in document
+             bool   descend)		// I - Descend into child node?
+{
+  if (descend && tree->child)
+  {
+    // Descend...
+    return (tree->child);
+  }
+  else if (tree == doc)
+  {
+    // At the top, stop...
+    return (NULL);
+  }
+  else if (tree->next)
+  {
+    // Visit sibling...
+    return (tree->next);
+  }
+  else if (tree->parent && tree->parent != doc)
+  {
+    // Ascend
+    tree = tree->parent;
+
+    while (tree && !tree->next)
+    {
+      // Get the next parent...
+      if (tree->parent == doc || !tree->parent)
+	tree = NULL;
+      else
+	tree = tree->parent;
+    }
+
+    // If we still have a node, resume at its sibling...
+    return (tree ? tree->next : NULL);
+  }
+  else
+  {
+    // No more...
+    return (NULL);
+  }
+}
+
+
 //
 // 'compare_variables()' - Compare two markup variables.
 //
@@ -2614,7 +2662,7 @@ insert_space(tree_t *parent,	// I - Parent node
 static int			// O - -1 on error, MARKUP_nnnn otherwise
 parse_markup(tree_t *t,		// I - Current tree entry
              FILE   *fp,	// I - Input file
-	     int    *linenum)	// O - Current line number
+	     long   *linenum)	// O - Current line number
 {
   int	ch, ch2;		// Characters from file
   uchar	markup[255],		// Markup string...
@@ -2627,6 +2675,7 @@ parse_markup(tree_t *t,		// I - Current tree entry
   mptr = markup;
 
   while ((ch = getc(fp)) != EOF && mptr < (markup + sizeof(markup) - 1))
+  {
     if (ch == '>' || isspace(ch))
       break;
     else if (ch == '/' && mptr > markup)
@@ -2657,6 +2706,7 @@ parse_markup(tree_t *t,		// I - Current tree entry
         break;
       }
     }
+  }
 
   *mptr = '\0';
 
@@ -2683,7 +2733,7 @@ parse_markup(tree_t *t,		// I - Current tree entry
     t->markup = (markup_t)((const char **)temp - _htmlMarkups);
     cptr      = comment;
 
-    DEBUG_printf(("%s%s, line %d\n", indent, markup, *linenum));
+    DEBUG_printf(("%s%s, line %ld\n", indent, markup, *linenum));
   }
 
   if (t->markup == MARKUP_COMMENT || t->markup == MARKUP_UNKNOWN)
@@ -2740,8 +2790,7 @@ parse_markup(tree_t *t,		// I - Current tree entry
 	  *eptr = '\0';
 	  if (!ch)
 	  {
-	    progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %d of %s.",
-	                   *linenum, _htmlCurrentFile);
+	    progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %ld of %s.", *linenum, _htmlCurrentFile);
 
             if (cptr < (comment + sizeof(comment) - 1))
 	      *cptr++ = '&';
@@ -2750,9 +2799,7 @@ parse_markup(tree_t *t,		// I - Current tree entry
 	  }
 	  else if ((ch = iso8859(entity)) == 0)
 	  {
-	    progress_error(HD_ERROR_HTML_ERROR,
-	                   "Unknown character entity \"&%s;\" on line %d of %s.",
-	                   entity, *linenum, _htmlCurrentFile);
+	    progress_error(HD_ERROR_HTML_ERROR, "Unknown character entity \"&%s;\" on line %ld of %s.", entity, *linenum, _htmlCurrentFile);
 
             if (cptr < (comment + sizeof(comment) - 1))
 	      *cptr++ = '&';
@@ -2823,7 +2870,7 @@ parse_markup(tree_t *t,		// I - Current tree entry
 static int				// O - -1 on error, 0 on success
 parse_variable(tree_t *t,		// I - Current tree entry
                FILE   *fp,		// I - Input file
-	       int    *linenum)		// I - Current line number
+	       long   *linenum)		// I - Current line number
 {
   uchar	name[1024],			// Name of variable
 	value[10240],			// Value of variable
@@ -2835,6 +2882,7 @@ parse_variable(tree_t *t,		// I - Current tree entry
 
   ptr = name;
   while ((ch = getc(fp)) != EOF)
+  {
     if (isspace(ch) || ch == '=' || ch == '>' || ch == '\r')
       break;
     else if (ch == '/' && ptr == name)
@@ -2850,6 +2898,7 @@ parse_variable(tree_t *t,		// I - Current tree entry
       if (ch)
         *ptr++ = (uchar)ch;
     }
+  }
 
   *ptr = '\0';
 
@@ -2907,9 +2956,7 @@ parse_variable(tree_t *t,		// I - Current tree entry
               *eptr = '\0';
               if (!ch)
 	      {
-		progress_error(HD_ERROR_HTML_ERROR,
-		               "Unquoted & on line %d of %s.",
-	                       *linenum, _htmlCurrentFile);
+		progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %ld of %s.", *linenum, _htmlCurrentFile);
 
         	if (ptr < (value + sizeof(value) - 1))
 		  *ptr++ = '&';
@@ -2918,9 +2965,7 @@ parse_variable(tree_t *t,		// I - Current tree entry
 	      }
 	      else if ((ch = iso8859(entity)) == 0)
 	      {
-		progress_error(HD_ERROR_HTML_ERROR,
-		               "Unknown character entity \"&%s;\" on line %d of %s.",
-	                       entity, *linenum, _htmlCurrentFile);
+		progress_error(HD_ERROR_HTML_ERROR, "Unknown character entity \"&%s;\" on line %ld of %s.", entity, *linenum, _htmlCurrentFile);
 
         	if (ptr < (value + sizeof(value) - 1))
 		  *ptr++ = '&';
@@ -2981,8 +3026,7 @@ parse_variable(tree_t *t,		// I - Current tree entry
               *eptr = '\0';
               if (!ch)
 	      {
-		progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %d of %s.",
-	                       *linenum, _htmlCurrentFile);
+		progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %ld of %s.", *linenum, _htmlCurrentFile);
 
         	if (ptr < (value + sizeof(value) - 1))
 		  *ptr++ = '&';
@@ -2991,9 +3035,7 @@ parse_variable(tree_t *t,		// I - Current tree entry
 	      }
 	      else if ((ch = iso8859(entity)) == 0)
 	      {
-		progress_error(HD_ERROR_HTML_ERROR,
-		               "Unknown character entity \"&%s;\" on line %d of %s.",
-	                       entity, *linenum, _htmlCurrentFile);
+		progress_error(HD_ERROR_HTML_ERROR, "Unknown character entity \"&%s;\" on line %ld of %s.", entity, *linenum, _htmlCurrentFile);
 
         	if (ptr < (value + sizeof(value) - 1))
 		  *ptr++ = '&';
@@ -3055,8 +3097,7 @@ parse_variable(tree_t *t,		// I - Current tree entry
               *eptr = '\0';
               if (!ch)
 	      {
-		progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %d of %s.",
-	                       *linenum, _htmlCurrentFile);
+		progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %ld of %s.", *linenum, _htmlCurrentFile);
 
         	if (ptr < (value + sizeof(value) - 1))
 		  *ptr++ = '&';
@@ -3065,9 +3106,7 @@ parse_variable(tree_t *t,		// I - Current tree entry
 	      }
 	      else if ((ch = iso8859(entity)) == 0)
 	      {
-		progress_error(HD_ERROR_HTML_ERROR,
-		               "Unknown character entity \"&%s;\" on line %d of %s.",
-	                       entity, *linenum, _htmlCurrentFile);
+		progress_error(HD_ERROR_HTML_ERROR, "Unknown character entity \"&%s;\" on line %ld of %s.", entity, *linenum, _htmlCurrentFile);
 
         	if (ptr < (value + sizeof(value) - 1))
 		  *ptr++ = '&';
@@ -3404,13 +3443,14 @@ fix_filename(char *filename,		// I - Original filename
       strncmp(filename, ".\\", 2) == 0)
     filename += 2;
 
+#ifdef HAVE_LIBCUPS
   if (strncmp(base, "http://", 7) == 0 || strncmp(base, "https://", 8) == 0)
   {
     // Base is a URL...
     char	scheme[32],		// URI scheme
 		userpass[256],		// Username:password
 		host[256],		// Hostname or IP address
-		resource[256];		// Resource path
+		resource[1024];		// Resource path
     int		port;			// Port number
 
     httpSeparateURI(HTTP_URI_CODING_ALL, base, scheme, sizeof(scheme), userpass, sizeof(userpass), host, sizeof(host), &port, resource, sizeof(resource));
@@ -3437,6 +3477,7 @@ fix_filename(char *filename,		// I - Original filename
     }
   }
   else
+#endif // HAVE_LIBCUPS
   {
     // Base is a filename...
     if (filename[0] == '/' || filename[0] == '\\' || base == NULL ||
@@ -3567,6 +3608,8 @@ htmlFixLinks(tree_t *doc,		// I - Top node
 	     uchar  *base)		// I - Base directory/path
 {
   uchar		*href;			// HREF attribute
+  int		num_bstack = 0;		// Number of base paths on the stack
+  uchar		*bstack[100];		// Stack of base paths
   char		full_href[1024];	// Full HREF value
   const char	*debug;			// HTMLDOC_DEBUG environment variable
   static int	show_debug = -1;	// Show debug messages?
@@ -3671,12 +3714,60 @@ htmlFixLinks(tree_t *doc,		// I - Top node
       }
     }
     else if (tree->markup == MARKUP_FILE)
+    {
+      // Save current base and get new base...
+      if (num_bstack < (int)(sizeof(bstack) / sizeof(bstack[0])))
+        bstack[num_bstack++] = base;
+
       base = htmlGetVariable(tree, (uchar *)"_HD_BASE");
+    }
 
+    // Walk to the next logical node in the tree...
     if (tree->child)
-      htmlFixLinks(doc, tree->child, base);
+    {
+      // Descend...
+      tree = tree->child;
+    }
+    else if (tree == doc)
+    {
+      // At the top, stop...
+      tree = NULL;
+    }
+    else if (tree->next)
+    {
+      // Visit sibling...
+      tree = tree->next;
+    }
+    else if (tree->parent && tree->parent != doc)
+    {
+      // Ascend
+      tree = tree->parent;
 
-    tree = tree->next;
+      while (tree && !tree->next)
+      {
+        if (tree->markup == MARKUP_FILE && num_bstack > 0)
+        {
+	  // Restore previous base...
+	  num_bstack --;
+	  base = bstack[num_bstack];
+	}
+
+        // Get the next parent...
+	if (tree->parent == doc || !tree->parent)
+	  tree = NULL;
+	else
+	  tree = tree->parent;
+      }
+
+      // If we still have a node, resume at its sibling...
+      if (tree)
+        tree = tree->next;
+    }
+    else
+    {
+      // No more...
+      tree = NULL;
+    }
   }
 }
 
@@ -3692,6 +3783,8 @@ utf8_getc(int  ch,                      // I - Initial character
   int  ch2 = -1, ch3 = -1;              // Temporary characters
 
 
+  start:
+
   if ((ch & 0xe0) == 0xc0)
   {
     // Two-byte sequence for 0x80 to 0x7ff...
@@ -3730,7 +3823,8 @@ utf8_getc(int  ch,                      // I - Initial character
     // them...  Try reading another character...
     //
     // TODO: Emit a warning about this...
-    return (utf8_getc(getc(fp), fp));
+    ch = getc(fp);
+    goto start;
   }
 
   return (htmlMapUnicode(ch));
@@ -3746,4 +3840,3 @@ utf8_getc(int  ch,                      // I - Initial character
 
   return (0);
 }
-
