@@ -1,41 +1,37 @@
-/*
- * Utility functions for HTMLDOC, a HTML document processing program.
- *
- * Copyright © 2011-2020 by Michael R Sweet.
- * Copyright © 1997-2010 by Easy Software Products.  All rights reserved.
- *
- * This program is free software.  Distribution and use rights are outlined in
- * the file "COPYING".
- */
-
-/*
- * Include necessary headers.
- */
+//
+// Utility functions for HTMLDOC, a HTML document processing program.
+//
+// Copyright © 2011-2024 by Michael R Sweet.
+// Copyright © 1997-2010 by Easy Software Products.  All rights reserved.
+//
+// This program is free software.  Distribution and use rights are outlined in
+// the file "COPYING".
+//
 
 #include "htmldoc.h"
 #include <ctype.h>
 
 
-/*
- * 'format_number()' - Format a number into arabic numerals, roman numerals,
- *                     or letters.
- */
+//
+// 'format_number()' - Format a number into arabic numerals, roman numerals,
+//                     or letters.
+//
 
-char *				/* O - String */
-format_number(int  n,		/* I - Number */
-              char f)		/* I - Format */
+char *					// O - String
+format_number(int  n,			// I - Number
+              char f)			// I - Format
 {
-  static const char *ones[10] =	/* Roman numerals, 0-9 */
+  static const char *ones[10] =		// Roman numerals, 0-9
 		{
 		  "",	"i",	"ii",	"iii",	"iv",
 		  "v",	"vi",	"vii",	"viii",	"ix"
 		},
-		*tens[10] =	/* Roman numerals, 10-90 */
+		*tens[10] =		// Roman numerals, 10-90
 		{
 		  "",	"x",	"xx",	"xxx",	"xl",
 		  "l",	"lx",	"lxx",	"lxxx",	"xc"
 		},
-		*hundreds[30] =	/* Roman numerals, 100-2900 */
+		*hundreds[30] =		// Roman numerals, 100-2900
 		{
 		  "",    "c",    "cc",    "ccc",    "cd",
 		  "d",   "dc",   "dcc",   "dccc",   "cm",
@@ -44,17 +40,17 @@ format_number(int  n,		/* I - Number */
 		  "mm",  "mmc",  "mmcc",  "mmccc",  "mmcd",
 		  "mmd", "mmdc", "mmdcc", "mmdccc", "mmcm"
 		};
-  static const char *ONES[10] =	/* Roman numerals, 0-9 */
+  static const char *ONES[10] =		// Roman numerals, 0-9
 		{
 		  "",	"I",	"II",	"III",	"IV",
 		  "V",	"VI",	"VII",	"VIII",	"IX"
 		},
-		*TENS[10] =	/* Roman numerals, 10-90 */
+		*TENS[10] =		// Roman numerals, 10-90
 		{
 		  "",	"X",	"XX",	"XXX",	"XL",
 		  "L",	"LX",	"LXX",	"LXXX",	"XC"
 		},
-		*HUNDREDS[30] =	/* Roman numerals, 100-2900 */
+		*HUNDREDS[30] =		// Roman numerals, 100-2900
 		{
 		  "",    "C",    "CC",    "CCC",    "CD",
 		  "D",   "DC",   "DCC",   "DCCC",   "CM",
@@ -63,7 +59,7 @@ format_number(int  n,		/* I - Number */
 		  "MM",  "MMC",  "MMCC",  "MMCCC",  "MMCD",
 		  "MMD", "MMDC", "MMDCC", "MMDCCC", "MMCM"
 		};
-  static char	buffer[1024];	/* String buffer */
+  static char	buffer[1024];		// String buffer
 
 
   switch (f)
@@ -115,30 +111,30 @@ format_number(int  n,		/* I - Number */
 }
 
 
-/*
- * 'get_color()' - Get a standard color value...
- */
+//
+// 'get_color()' - Get a standard color value...
+//
 
 void
-get_color(const uchar *color,	/* I - Color attribute */
-          float       *rgb,	/* O - RGB value */
-	  int         defblack)	/* I - Default color is black? */
+get_color(const uchar *color,	// I - Color attribute
+          float       *rgb,	// O - RGB value
+	  int         defblack)	// I - Default color is black?
 {
-  int		i;		/* Looping vars */
-  static uchar	tempcolor[8];	/* Temporary holding place for hex colors */
+  int		i;		// Looping vars
+  static uchar	tempcolor[8];	// Temporary holding place for hex colors
   static struct
   {
-    const char	*name;		/* Color name */
-    uchar	red,		/* Red value */
-		green,		/* Green value */
-		blue;		/* Blue value */
-  }		colors[] =	/* Color "database" */
+    const char	*name;		// Color name
+    uchar	red,		// Red value
+		green,		// Green value
+		blue;		// Blue value
+  }		colors[] =	// Color "database"
   {
-    { "aqua",		0,   255, 255 }, /* AKA Cyan */
+    { "aqua",		0,   255, 255 }, // AKA Cyan
     { "black",		0,   0,   0 },
     { "blue",		0,   0,   255 },
     { "cyan",		0,   255, 255 },
-    { "fuchsia",	255, 0,   255 }, /* AKA Magenta */
+    { "fuchsia",	255, 0,   255 }, // AKA Magenta
     { "gray",		128, 128, 128 },
     { "green",		0,   128, 0 },
     { "grey",		128, 128, 128 },
@@ -190,10 +186,7 @@ get_color(const uchar *color,	/* I - Color attribute */
   }
   else if (color[0] == '#')
   {
-   /*
-    * RGB value in hex...
-    */
-
+    // RGB value in hex...
     i = (int)strlen((char *)color + 1);
     if (i == 3)
     {
@@ -227,9 +220,9 @@ get_color(const uchar *color,	/* I - Color attribute */
     if (i >= (int)(sizeof(colors) / sizeof(colors[0])))
     {
       if (defblack)
-        i = 1; /* Black */
+        i = 1; // Black
       else
-        i = 17; /* White */
+        i = 17; // White
     }
 
     rgb[0] = colors[i].red / 255.0f;
@@ -252,80 +245,74 @@ get_format(const char *fmt,		// I - Old "fff" format
 
   for (i = 0; i < 3; i ++)
   {
-    if (formats[i])
-    {
-      free(formats[i]);
-      formats[i] = NULL;
-    }
-
     switch (fmt[i])
     {
       case '/' :
-          formats[i] = strdup("$PAGE(1)/$PAGES");
+          formats[i] = hd_strdup("$PAGE(1)/$PAGES");
           break;
 
       case ':' :
-          formats[i] = strdup("$CHAPTERPAGE(1)/$CHAPTERPAGES");
+          formats[i] = hd_strdup("$CHAPTERPAGE(1)/$CHAPTERPAGES");
           break;
 
       case '1' :
-          formats[i] = strdup("$PAGE(1)");
+          formats[i] = hd_strdup("$PAGE(1)");
           break;
 
       case 'a' :
-          formats[i] = strdup("$PAGE(a)");
+          formats[i] = hd_strdup("$PAGE(a)");
           break;
 
       case 'A' :
-          formats[i] = strdup("$PAGE(A)");
+          formats[i] = hd_strdup("$PAGE(A)");
           break;
 
       case 'c' :
-          formats[i] = strdup("$CHAPTER");
+          formats[i] = hd_strdup("$CHAPTER");
           break;
 
       case 'C' :
-          formats[i] = strdup("$CHAPTERPAGE(1)");
+          formats[i] = hd_strdup("$CHAPTERPAGE(1)");
           break;
 
       case 'd' :
-          formats[i] = strdup("$DATE");
+          formats[i] = hd_strdup("$DATE");
           break;
 
       case 'D' :
-          formats[i] = strdup("$DATE $TIME");
+          formats[i] = hd_strdup("$DATE $TIME");
           break;
 
       case 'h' :
-          formats[i] = strdup("$HEADING");
+          formats[i] = hd_strdup("$HEADING");
           break;
 
       case 'i' :
-          formats[i] = strdup("$PAGE(i)");
+          formats[i] = hd_strdup("$PAGE(i)");
           break;
 
       case 'I' :
-          formats[i] = strdup("$PAGE(I)");
+          formats[i] = hd_strdup("$PAGE(I)");
           break;
 
       case 'l' :
-          formats[i] = strdup("$LOGOIMAGE");
+          formats[i] = hd_strdup("$LOGOIMAGE");
           break;
 
       case 'L' :
-          formats[i] = strdup("$LETTERHEAD");
+          formats[i] = hd_strdup("$LETTERHEAD");
           break;
 
       case 't' :
-          formats[i] = strdup("$TITLE");
+          formats[i] = hd_strdup("$TITLE");
           break;
 
       case 'T' :
-          formats[i] = strdup("$TIME");
+          formats[i] = hd_strdup("$TIME");
           break;
 
       case 'u' :
-          formats[i] = strdup("$URL");
+          formats[i] = hd_strdup("$URL");
           break;
 
       default :
@@ -373,42 +360,42 @@ get_fmt(char **formats)			// I - New format strings
   strlcpy(fmt, "...", sizeof(fmt));
 
   for (i = 0; i < 3; i ++)
+  {
     if (formats[i])
+    {
       for (j = 0; j < (int)(sizeof(table) / sizeof(table[0])); j ++)
+      {
         if (strcmp(formats[i], table[j].format) == 0)
 	{
 	  fmt[i] = table[j].f;
 	  break;
 	}
+      }
+    }
+  }
 
   return (fmt);
 }
 
 
-/*
- * 'get_measurement()' - Get a size measurement in inches, points, centimeters,
- *                       or millimeters.
- */
+//
+// 'get_measurement()' - Get a size measurement in inches, points, centimeters,
+//                       or millimeters.
+//
 
-int				/* O - Measurement in points */
-get_measurement(const char *s,	/* I - Measurement string */
-                float      mul)	/* I - Multiplier */
+int					// O - Measurement in points
+get_measurement(const char *s,		// I - Measurement string
+                float      mul)		// I - Multiplier
 {
-  float	val;			/* Measurement value */
+  float	val;				// Measurement value
 
 
- /*
-  * Get the floating point value of "s" and skip all digits and decimal points.
-  */
-
+  // Get the floating point value of "s" and skip all digits and decimal points.
   val = (float)atof(s);
   while (isdigit(*s) || *s == '.')
     s ++;
 
- /*
-  * Check for a trailing unit specifier...
-  */
-
+  // Check for a trailing unit specifier...
   if (strcasecmp(s, "mm") == 0)
     val *= 72.0f / 25.4f;
   else if (strcasecmp(s, "cm") == 0)
@@ -422,95 +409,69 @@ get_measurement(const char *s,	/* I - Measurement string */
 }
 
 
-/*
- * 'set_page_size()' - Set the output page size.
- */
+//
+// 'set_page_size()' - Set the output page size.
+//
 
 void
-set_page_size(const char *size)	/* I - Page size string */
+set_page_size(const char *size)		// I - Page size string
 {
-  float	width,			/* Width in points */
-	length;			/* Length in points */
-  char	units[255];		/* Units string */
+  float	width,				// Width in points
+	length;				// Length in points
+  char	units[255];			// Units string
 
 
- /*
-  * Check for common media sizes...
-  */
-
-  if (strcasecmp(size, "letter") == 0 ||
-      strcasecmp(size, "a") == 0)
+  // Check for common media sizes...
+  if (!strcasecmp(size, "letter") || !strcasecmp(size, "a"))
   {
-   /*
-    * US Letter - 8.5x11 inches (216x279mm).
-    */
-
+    // US Letter - 8.5x11 inches (216x279mm).
     PageWidth  = 612;
     PageLength = 792;
   }
-  else if (strcasecmp(size, "legal") == 0)
+  else if (!strcasecmp(size, "legal"))
   {
-   /*
-    * US Legal - 8.5x14 inches (216x356mm).
-    */
-
+    // US Legal - 8.5x14 inches (216x356mm).
     PageWidth  = 612;
     PageLength = 1008;
   }
-  else if (strcasecmp(size, "tabloid") == 0 ||
-           strcasecmp(size, "b") == 0)
+  else if (!strcasecmp(size, "tabloid") || !strcasecmp(size, "b"))
   {
-   /*
-    * US Tabloid - 11x17 inches (279x432mm).
-    */
-
+    // US Tabloid - 11x17 inches (279x432mm).
     PageWidth  = 792;
     PageLength = 1224;
   }
-  else if (strcasecmp(size, "a4") == 0)
+  else if (!strcasecmp(size, "a4"))
   {
-   /*
-    * European standard A4 - 210x297mm (8.27x11.69 inches).
-    */
-
+    // European standard A4 - 210x297mm (8.27x11.69 inches).
     PageWidth  = 595;
     PageLength = 842;
   }
-  else if (strcasecmp(size, "a3") == 0)
+  else if (!strcasecmp(size, "a3"))
   {
-   /*
-    * European standard A3 - 297x420mm (11.69x16.54 inches).
-    */
-
+    // European standard A3 - 297x420mm (11.69x16.54 inches).
     PageWidth  = 842;
     PageLength = 1190;
   }
-  else if (strcasecmp(size, "universal") == 0)
+  else if (!strcasecmp(size, "universal"))
   {
-   /*
-    * "Universal" size - 8.27x11.00 inches (210x279mm).
-    */
-
+    // "Universal" size - 8.27x11.00 inches (210x279mm).
     PageWidth  = 595;
     PageLength = 792;
   }
   else if (sscanf(size, "%fx%f%254s", &width, &length, units) >= 2)
   {
-   /*
-    * Custom size...
-    */
-
-    if (strcasecmp(units, "mm") == 0)
+    // Custom size...
+    if (!strcasecmp(units, "mm"))
     {
       PageWidth  = (int)(72.0 * width / 25.4);
       PageLength = (int)(72.0 * length / 25.4);
     }
-    else if (strcasecmp(units, "cm") == 0)
+    else if (!strcasecmp(units, "cm"))
     {
       PageWidth  = (int)(72.0 * width / 2.54);
       PageLength = (int)(72.0 * length / 2.54);
     }
-    else if (strncasecmp(units, "in", 2) == 0)
+    else if (!strncasecmp(units, "in", 2))
     {
       PageWidth  = (int)(72.0 * width);
       PageLength = (int)(72.0 * length);
