@@ -86,10 +86,19 @@ typedef __int64 ssize_t;
 
 
 /*
- * What is the version number for this software?
+ * What is the version number and year for this software?
  */
 
-#define SVERSION	"1.9.21"
+#define SVERSION "1.9.22"
+#define SYEAR "2025"
+
+
+/*
+ * Limits for input processing...
+ */
+
+#define MAX_DEPTH	1000	/* Maximum depth of document tree */
+#define MAX_INCLUDES	32	/* Maximum levels of included/embedded files */
 
 
 /*
@@ -122,7 +131,14 @@ typedef __int64 ssize_t;
 
 
 /*
- * Do we have the FLTK library?
+ * Have the CUPS library?
+ */
+
+#define HAVE_LIBCUPS 1
+
+
+/*
+ * Have the FLTK library?
  */
 
 #ifndef _CONSOLE
@@ -131,42 +147,42 @@ typedef __int64 ssize_t;
 
 
 /*
- * Do we have the JPEG library?
+ * Have the JPEG library?
  */
 
 #define HAVE_LIBJPEG 1
 
 
 /*
- * Do we have the PNG library?
+ * Have the PNG library?
  */
 
 #define HAVE_LIBPNG 1
 
 
 /*
- * Do we have the Xpm library?
+ * Have the Xpm library?
  */
 
 /* #undef HAVE_LIBXPM */
 
 
 /*
- * Do we need to use <strings.h>?
+ * Need to use <strings.h>?
  */
 
 /* #undef HAVE_STRINGS_H */
 
 
 /*
- * Do we have the <locale.h> header file?
+ * Have the <locale.h> header file?
  */
 
 #define HAVE_LOCALE_H 1
 
 
 /*
- * Do we have some of the "standard" string functions?
+ * Have some of the "standard" string functions?
  */
 
 #define HAVE_STRDUP 1
@@ -174,12 +190,6 @@ typedef __int64 ssize_t;
 #define HAVE_STRNCASECMP 1
 /* #undef HAVE_STRLCAT */
 /* #undef HAVE_STRLCPY */
-
-
-/*
- * How about snprintf() and vsnprintf()?
- */
-
 #define HAVE_SNPRINTF 1
 #define HAVE_VSNPRINTF 1
 
